@@ -1,4 +1,4 @@
-/* $Id: ammonite-general.c,v 1.14 2001/01/10 06:28:52 mfleming Exp $
+/* $Id: ammonite-general.c,v 1.16 2001/02/22 02:10:43 mstachow Exp $
  * 
  * ammonite-general.c:
  * 
@@ -359,6 +359,31 @@ ammonite_atexit (void)
 	}
 }
 
+#define KEY_GCONF_TRILOBITE_MACHINE_ID "/apps/eazel-trilobite/machine-id"
+
+char *
+ammonite_get_machine_id ()
+{
+	GConfValue *val = NULL;
+	char *machine_id;
+
+	val = gconf_engine_get (gl_engine_gconf, KEY_GCONF_TRILOBITE_MACHINE_ID, NULL);
+
+	if (NULL == val || GCONF_VALUE_STRING != val->type ) {
+	        machine_id = NULL;
+	} else {
+		machine_id = g_strdup (gconf_value_get_string (val));
+	}	
+
+	if (val != NULL) {
+		gconf_value_free (val);
+	}
+
+	return machine_id;
+}
+
+
+
 gboolean
 ammonite_init (PortableServer_POA poa)
 {
@@ -586,7 +611,7 @@ ammonite_fail_code_to_string (CORBA_long code)
  */
 
 char *
-ammonite_get_default_user_username (EazelProxy_UserControl user_control)
+ammonite_get_default_user_username (void)
 {
 	CORBA_Environment       ev;
 	EazelProxy_User         *user;
@@ -594,9 +619,9 @@ ammonite_get_default_user_username (EazelProxy_UserControl user_control)
 
 	CORBA_exception_init (&ev);
 
-	if (CORBA_OBJECT_NIL != user_control) {
+	if (CORBA_OBJECT_NIL != gl_user_control) {
 
-		user = EazelProxy_UserControl_get_default_user (user_control, &ev);
+		user = EazelProxy_UserControl_get_default_user (gl_user_control, &ev);
 
 		if (CORBA_NO_EXCEPTION != ev._major) {
 			DEBUG_MSG (("No Eazel Service User is currently logged in"));
@@ -606,6 +631,8 @@ ammonite_get_default_user_username (EazelProxy_UserControl user_control)
 			ret = g_strdup (user->user_name);
 			CORBA_free (user);
 		}
+	} else {
+		g_warning ("libammonite: called before ammonite_init\n");
 	}
 
 	CORBA_exception_free (&ev);
