@@ -1,4 +1,4 @@
-/* $Id: authenticate.c,v 1.13 2001/01/22 08:53:32 yakk Exp $
+/* $Id: authenticate.c,v 1.16 2001/02/22 01:25:02 mfleming Exp $
  * 
  * Copyright (C) 2000 Eazel, Inc
  *
@@ -76,10 +76,11 @@ typedef struct {
  *******************************************************************/
 
 /* from impl-eazelproxy.c */
-extern gchar * gl_user_level;
+extern gchar *gl_user_level;
 extern gboolean gl_user_level_have;
-extern gchar * gl_machine_name;
-extern gboolean gl_machine_name_have;
+extern gchar *gl_machine_id;
+extern gboolean gl_machine_id_have;
+extern gchar *gl_distribution_name;
 
 static GList * 
 authenticate_add_default_headers (GList *header_list)
@@ -91,9 +92,15 @@ authenticate_add_default_headers (GList *header_list)
 			      );
 	}
 
-	if (gl_machine_name_have) {
+	if (gl_machine_id_have) {
 		header_list = g_list_append (header_list, 
-				g_strdup_printf ("X-Eazel-Machine-Name: %s", gl_machine_name)
+				g_strdup_printf ("X-Eazel-Machine-ID: %s", gl_machine_id)
+			      );
+	}
+
+	if (gl_distribution_name != NULL) {
+		header_list = g_list_append (header_list, 
+				g_strdup_printf ("X-Eazel-Distro-Name: %s", gl_distribution_name)
 			      );
 	}
 
@@ -444,6 +451,7 @@ authenticate_http_callback (gpointer user_data, Socket *sock, gboolean success)
 	switch (state->state) {
 		case AUTHN_HTTP0:
 			if (!success) {
+				log ("ERROR: Authenticate failed: HTTP request 0 failed");
 				authenticate_finish_failed (state, EAZELPROXY_AUTHN_FAIL_NETWORK, NULL);
 			} else {
 				http_connection_read (sock, state, authenticate_http_read_callback);
@@ -452,6 +460,7 @@ authenticate_http_callback (gpointer user_data, Socket *sock, gboolean success)
 
 		case AUTHN_HTTP1:
 			if (!success) {
+				log ("ERROR: Authenticate failed: HTTP request 1 failed");
 				authenticate_finish_failed (state, EAZELPROXY_AUTHN_FAIL_NETWORK, NULL);
 			} else {
 				http_connection_read (sock, state, authenticate_http_read_callback);
