@@ -1,4 +1,4 @@
-/* $Id: http-connection.c,v 1.23 2001/01/16 21:30:08 mfleming Exp $
+/* $Id: http-connection.c,v 1.24 2001/02/23 12:23:35 seth Exp $
  * 
  * Copyright (C) 2000 Eazel, Inc
  *
@@ -296,7 +296,7 @@ http_parse_authn_header ( /*INOUT*/ char **p_header, /*OUT*/ char **p_key, /*OUT
 	*p_value = NULL;
 
 	/* Eat whitespace and seperators */
-	for ( ; *current && (isspace (*current) || ',' == *current || '=' == *current) ; current++) ;
+	for ( ; *current && (isspace ((guchar) *current) || ',' == *current || '=' == *current) ; current++) ;
 
 	if ( ! *current) {
 		/* all done */
@@ -306,7 +306,7 @@ http_parse_authn_header ( /*INOUT*/ char **p_header, /*OUT*/ char **p_key, /*OUT
 	*p_key = current;
 
 	/* Get key */
-	for ( ; *current && ! (isspace (*current) || ',' == *current || '=' == *current) ; current++) ;
+	for ( ; *current && ! (isspace ((guchar) *current) || ',' == *current || '=' == *current) ; current++) ;
 
 	if ( '=' == *current ) {
 		*current++ = '\0';
@@ -322,7 +322,7 @@ http_parse_authn_header ( /*INOUT*/ char **p_header, /*OUT*/ char **p_key, /*OUT
 			}
 		} else {
 			*p_value = current;
-			for ( ; *current && ! ( isspace (*current) || ',' == * current || ';' == *current) ; current++ );
+			for ( ; *current && ! ( isspace ((guchar) *current) || ',' == * current || ';' == *current) ; current++ );
 			*current++ = '\0';
 			if ( '\0' == *p_value ) {
 				*p_value = NULL;
@@ -397,8 +397,8 @@ http_connection_connect (
 		if ( NULL != fm_mirror_arg ) {
 			socket_set_mirrors (server_socket, fm_mirror_arg, to_mirror_arg);
 		} else {
-			fm_mirror_filename = g_strdup_printf ("%s/ep-%d-%d-fm-http-req.log", config.mirror_log_dir, getpid(), http_req_count);
-			to_mirror_filename = g_strdup_printf ("%s/ep-%d-%d-to-http-req.log", config.mirror_log_dir, getpid(), http_req_count);
+			fm_mirror_filename = g_strdup_printf ("%s/ep-%ld-%d-fm-http-req.log", config.mirror_log_dir, (long int)getpid(), http_req_count);
+			to_mirror_filename = g_strdup_printf ("%s/ep-%ld-%d-to-http-req.log", config.mirror_log_dir, (long int)getpid(), http_req_count);
 			fm_mirror = fopen (fm_mirror_filename, "w");
 			to_mirror = fopen (to_mirror_filename, "w");
 			socket_set_mirrors (server_socket, fm_mirror, to_mirror);
