@@ -1,5 +1,5 @@
 
-/* $Id: impl-eazelproxy.c,v 1.66 2001/01/22 08:53:32 yakk Exp $
+/* $Id: impl-eazelproxy.c,v 1.70 2001/02/22 01:25:02 mfleming Exp $
  * 
  * Copyright (C) 2000 Eazel, Inc
  *
@@ -47,6 +47,10 @@
 #include "util-gconf.h"
 #include "eazelproxy-change-password.h"
 
+#include "trilobite-core-distribution.h"
+
+#include "uuid.h"
+
 /*******************************************************************
  * Types, Global Variables
  *******************************************************************/
@@ -54,7 +58,7 @@
 #define AUTHN_PROMPT_RETRIES 2		/* three prompts total */
 
 #define KEY_GCONF_NAUTILUS_USER_LEVEL "/apps/nautilus/user_level"
-#define KEY_GCONF_EAZEL_MACHINE_NAME "/apps/eazel-trilobite/machine_name"
+#define KEY_GCONF_TRILOBITE_MACHINE_ID "/apps/eazel-trilobite/machine-id"
 
 #define KEY_GCONF_DEFAULT_SERVICES_URI "/apps/eazel-trilobite/services-host"
 #define KEY_GCONF_DEFAULT_LOGIN_PATH "/apps/eazel-trilobite/default-services-login-path"
@@ -73,6 +77,8 @@ static User *gl_user_default 			= NULL;
 static GList *gl_user_list 			= NULL;
 /* The list of UserListener's */
 static GList *gl_user_listener_list 		= NULL;
+/* The list of AuthnCallbackCancelInfo's */
+static GList *gl_callback_cancel_list		= NULL;
 
 #ifdef ENABLE_REAUTHN
 /* The PID of eazel-proxy-util -d, for the default reauthn dialog */
@@ -86,19 +92,21 @@ static pid_t gl_util_pid = 0;
 gchar * gl_user_level;
 gboolean gl_user_level_have 		= FALSE;
 
-/* The current Eazel Services Machine Name setting */
+/* The current Eazel Services Machine id setting */
 /* exported for authenticate_add_default_headers:eazelproxy-authn.c */
-gchar * gl_machine_name;
-gboolean gl_machine_name_have 		= FALSE;
+gchar * gl_machine_id;
+gboolean gl_machine_id_have 		= FALSE;
+
+/* The current OS distribution name, if available */
+gchar * gl_distribution_name;
 
 /* Default ammonite settings */
 static gchar * gl_default_services_uri		= NULL;
 static gchar * gl_default_login_path		= NULL;
-static gchar * gl_default_service_username	= NULL;
 
 struct User {
 	EazelProxy_User user;
-	GList *callback_list;
+	GList *callback_list;		/* List of EazelProxy_AuthnCallback's */
 	DigestState *	p_digest;
 	guint32	authn_count;		/* Increments when user is reauthned; so that 
 					 * requests know when the user was reauthned
@@ -107,11 +115,20 @@ struct User {
 	EazelProxy_UserPrompter	prompter;	/* Present if user was created through
 						 * a _prompt_ method
 						 */
+					/* The prompter's authn callback, for cancellation.
+					 * Note that this same object is in callback_list as well
+					 */
+	EazelProxy_AuthnCallback prompter_authn_callback;
+
 	guint32 prompt_count;		/* Number of times user has been prompted */
 	gboolean prompt_username_specified; 	/* TRUE if a call to prompt_authenticate_user */
 						/* originally specified a username */ 
 };
 
+typedef struct {
+	EazelProxy_AuthnCallback callback;
+	gboolean has_canceled;
+} AuthnCallbackCancelInfo;
 
 
 /*******************************************************************
@@ -133,22 +150,6 @@ watch_gconf_user_level_cb (
 	}
 }
 
-
-static void /* UtilGConfCb */
-watch_gconf_machine_name_cb (
-	const UtilGConfWatchVariable *watched, 
-	const GConfValue *new_value
-) {
-	if (NULL == new_value) {
-		gl_machine_name_have = FALSE;
-		log ("machine name unset", gconf_value_get_string (new_value));
-	} else if (GCONF_VALUE_STRING == new_value->type ) {
-		gl_machine_name_have = TRUE;
-		log ("machine name changed to '%s'", gconf_value_get_string (new_value));
-	}
-}
-
-
 static void /* UtilGConfCb */
 watch_gconf_services_uri_cb (
 	const UtilGConfWatchVariable *watched, 
@@ -189,23 +190,61 @@ watch_gconf_services_uri_cb (
 	}
 }
 
+#define UUID_STRING_LEN 35
+
+static void
+generate_and_set_machine_id (void)
+{
+	uuid_t machine_id;
+	char machine_id_string [UUID_STRING_LEN];
+
+	uuid_generate (machine_id);
+	uuid_unparse (machine_id, machine_id_string);
+
+	g_free (gl_machine_id);
+	gl_machine_id = g_strdup (machine_id_string);
+	gl_machine_id_have = TRUE;
+
+	gconf_engine_set_string (gl_gconf_engine, KEY_GCONF_TRILOBITE_MACHINE_ID, machine_id_string, NULL);
+	log ("new machine id, '%s'", machine_id_string);
+}
+
+static void
+configure_machine_id (void)
+{
+	GConfValue *val = NULL;
+
+	val = gconf_engine_get (gl_gconf_engine, KEY_GCONF_TRILOBITE_MACHINE_ID, NULL);
+
+	if (NULL == val || GCONF_VALUE_STRING != val->type ) {
+		generate_and_set_machine_id ();
+	} else {
+		gl_machine_id = g_strdup (gconf_value_get_string (val));
+		gl_machine_id_have = TRUE;
+		log ("machine id is '%s'", gl_machine_id);
+	}	
+
+	if (val != NULL) {
+		gconf_value_free (val);
+	}
+}
+
+
 static void
 get_defaults_from_gconf ()
 {
-	static const UtilGConfWatchVariable to_watch[] = {
-		/* Nautilus user level -- so it can be appended to the proxied request headers */
-		{KEY_GCONF_NAUTILUS_USER_LEVEL, GCONF_VALUE_STRING, {(gchar **)&gl_user_level}, watch_gconf_user_level_cb},
-		{KEY_GCONF_EAZEL_MACHINE_NAME, GCONF_VALUE_STRING, {(gchar **)&gl_machine_name}, watch_gconf_machine_name_cb},
-		/* Default services URI--where incoming HTTP requests should be redirected  */
-		{KEY_GCONF_DEFAULT_SERVICES_URI, GCONF_VALUE_STRING, {(gchar **)&gl_default_services_uri}, watch_gconf_services_uri_cb},
+	/* Nautilus user level -- so it can be appended to the proxied request headers */
+	static const UtilGConfWatchVariable watch_user_level =
+		{KEY_GCONF_NAUTILUS_USER_LEVEL, GCONF_VALUE_STRING, {(gchar **)&gl_user_level}, watch_gconf_user_level_cb};
+	/* Default services URI--where incoming HTTP requests should be redirected  */
+	static const UtilGConfWatchVariable watch_services_uri =
+		{KEY_GCONF_DEFAULT_SERVICES_URI, GCONF_VALUE_STRING, {(gchar **)&gl_default_services_uri}, watch_gconf_services_uri_cb};
+	static const UtilGConfWatchVariable watch_login_path =
 		/* Default login path -- path below services URI where authn requests should be pointed */
-		{KEY_GCONF_DEFAULT_LOGIN_PATH, GCONF_VALUE_STRING, {(gchar **)&gl_default_login_path}, NULL},
-		/* Default eazel service user */
-		{KEY_GCONF_DEFAULT_USER, GCONF_VALUE_STRING, {(gchar **)&gl_default_service_username}, NULL}
-	};
+		{KEY_GCONF_DEFAULT_LOGIN_PATH, GCONF_VALUE_STRING, {(gchar **)&gl_default_login_path}, NULL};
 
 	/* Nautilus User Level */
-	util_gconf_watch_variable (to_watch + 0);
+	util_gconf_watch_variable (&watch_user_level);
 
 	/* Default Services URI */
 	/*
@@ -232,19 +271,124 @@ get_defaults_from_gconf ()
 		} else {
 			gl_default_services_uri = g_strdup (DEFAULT_SERVICES_URI);
 		}
-		util_gconf_watch_variable (to_watch + 1);
+		util_gconf_watch_variable (&watch_services_uri);
 	}
 
 	/* login path */
 	gl_default_login_path = g_strdup (DEFAULT_LOGIN_PATH);
-	util_gconf_watch_variable (to_watch + 2);
-
-	/* Default Eazel Service User */
-	util_gconf_watch_variable (to_watch + 3);
+	util_gconf_watch_variable (&watch_login_path);
 
+	
+	configure_machine_id();
 }
 
+/*******************************************************************
+ * Cancellation tracking stuff
+ * 
+ *******************************************************************/
+static gint /* GCompareFunc */
+glist_find_cancel_callback_info (gconstpointer a, gconstpointer b)
+{
+	EazelProxy_AuthnCallback callback_to_find;
+	const AuthnCallbackCancelInfo *info;
+	CORBA_Environment ev;
+	gint ret;
 
+	CORBA_exception_init (&ev);
+
+	info = (AuthnCallbackCancelInfo *)a;
+	callback_to_find = (EazelProxy_AuthnCallback)b;
+
+	ret = CORBA_Object_is_equivalent (info->callback, callback_to_find, &ev) ? 0 : -1;
+	
+	CORBA_exception_free (&ev);
+
+	return ret;
+}
+
+void
+callback_cancel_add (const EazelProxy_AuthnCallback callback)
+{
+	GList *list_item;
+	AuthnCallbackCancelInfo *new_info;
+	CORBA_Environment ev;
+
+	CORBA_exception_init (&ev);
+
+	list_item = g_list_find_custom (gl_callback_cancel_list, (gpointer)callback, glist_find_cancel_callback_info);
+
+	if (list_item) {
+		log ("Warning: identical callback objects used simultainously");
+	}
+
+	new_info = g_new0 (AuthnCallbackCancelInfo, 1);
+
+	new_info->callback = (EazelProxy_AuthnCallback)CORBA_Object_duplicate (callback, &ev);
+
+	gl_callback_cancel_list	= g_list_prepend (gl_callback_cancel_list, new_info);
+
+	CORBA_exception_free (&ev);
+}
+
+void
+callback_cancel_remove (const EazelProxy_AuthnCallback callback)
+{
+	GList *list_item;
+	CORBA_Environment ev;
+	AuthnCallbackCancelInfo *info;
+
+	CORBA_exception_init (&ev);
+
+	list_item = g_list_find_custom (gl_callback_cancel_list, (gpointer)callback, glist_find_cancel_callback_info);
+
+	if (list_item) {
+		info = (AuthnCallbackCancelInfo *) list_item->data;
+		
+		CORBA_Object_release (info->callback, &ev);
+		g_free (info);
+
+		gl_callback_cancel_list = g_list_remove_link (gl_callback_cancel_list, list_item);
+		g_list_free (list_item);
+	} else {
+		g_assert_not_reached();
+	}
+
+	CORBA_exception_free (&ev);
+}
+
+void
+callback_cancel_set_canceled (const EazelProxy_AuthnCallback callback)
+{
+	GList *list_item;
+	AuthnCallbackCancelInfo *info;
+
+	list_item = g_list_find_custom (gl_callback_cancel_list, (gpointer)callback, glist_find_cancel_callback_info);
+
+	if (list_item) {
+		info = (AuthnCallbackCancelInfo *) list_item->data;
+		info->has_canceled = TRUE;
+	} else {
+		g_assert_not_reached();
+	}
+}
+
+gboolean 
+callback_cancel_has_canceled (const EazelProxy_AuthnCallback callback)
+{
+	GList *list_item;
+	AuthnCallbackCancelInfo *info;
+
+	list_item = g_list_find_custom (gl_callback_cancel_list, (gpointer)callback, glist_find_cancel_callback_info);
+
+	if (list_item) {
+		info = (AuthnCallbackCancelInfo *) list_item->data;
+		return info->has_canceled;
+	}
+
+	g_assert_not_reached();
+	
+	return TRUE;
+}
 
 
 /*******************************************************************
@@ -283,6 +427,13 @@ impl_EazelProxy_UserControl_set_new_user_password (
 	CORBA_Environment * ev
 );
 
+static void
+impl_EazelProxy_UserControl_cancel_async_operation (
+	PortableServer_Servant servant,
+	const EazelProxy_AuthnCallback callback,
+	CORBA_Environment * ev
+);
+
 static CORBA_char *
 impl_EazelProxy_UserControl_get_authn_header_for_port (
 	PortableServer_Servant servant,
@@ -326,6 +477,7 @@ static POA_EazelProxy_UserControl__epv impl_EazelProxy_UserControl_epv = {
 	impl_EazelProxy_UserControl_prompt_authenticate_user,
 	impl_EazelProxy_UserControl_authenticate_user,
 	impl_EazelProxy_UserControl_set_new_user_password,
+	impl_EazelProxy_UserControl_cancel_async_operation,
 	impl_EazelProxy_UserControl_get_authn_header_for_port,
 	impl_EazelProxy_UserControl_logout_user,
 	impl_EazelProxy_UserControl_add_listener,
@@ -349,6 +501,15 @@ init_impl_eazelproxy (CORBA_ORB orb)
 	/* listen on important variables */
 	get_defaults_from_gconf();
 
+	gl_distribution_name = trilobite_get_distribution_name (trilobite_get_distribution(), TRUE, TRUE);
+
+	if (gl_distribution_name != NULL) {
+		log ("OS distribution is '%s'", gl_distribution_name);
+	} else {
+		log ("OS distribution cannot be found");
+
+	}
+
 	/* register our class */
 	POA_EazelProxy_UserControl__init(&impl_EazelProxy_UserControl_servant, &ev);
 	poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
@@ -711,11 +872,13 @@ user_callback_list_free (GList **p_list)
 	CORBA_exception_init (&ev);
 
 	g_return_if_fail ( NULL != p_list);
+
 	
 	for (	list_node = g_list_first (*p_list) ;
 		NULL != list_node ;
 		list_node = g_list_next (list_node)
 	) {
+		callback_cancel_remove ((EazelProxy_AuthnCallback)list_node->data);
 		CORBA_Object_release ((CORBA_Object)list_node->data, &ev);
 	}
 	g_list_free (*p_list);
@@ -852,6 +1015,7 @@ user_authenticate_cb (
 	if (!success) {
 		if ( 	CORBA_OBJECT_NIL != user->prompter
 			&& fail_info && fail_info->code == EAZELPROXY_AUTHN_FAIL_AUTHN
+			&& !callback_cancel_has_canceled (user->prompter_authn_callback)
 		) {
 			/* If we're in prompt_authenticate mode and we failed due to an AUTHN failure */
 			if (user->prompt_count >= AUTHN_PROMPT_RETRIES) {
@@ -1053,8 +1217,12 @@ user_proxy_request_cb (gpointer user_data, unsigned short port, HTTPRequestLine
 		*p_header_list = g_list_append (*p_header_list, g_strdup_printf ("X-Eazel-User-Level: %s", gl_user_level));
 	}
 
-	if (gl_machine_name_have) {
-		*p_header_list = g_list_append (*p_header_list, g_strdup_printf ("X-Eazel-Machine-Name: %s", gl_machine_name));
+	if (gl_machine_id_have) {
+		*p_header_list = g_list_append (*p_header_list, g_strdup_printf ("X-Eazel-Machine-ID: %s", gl_machine_id));
+	}
+
+	if (gl_distribution_name != NULL) {
+		*p_header_list = g_list_append (*p_header_list, g_strdup_printf ("X-Eazel-Distro-Name: %s", gl_distribution_name));
 	}
 
 	return GUINT_TO_POINTER (user->authn_count);
@@ -1340,7 +1508,7 @@ impl_EazelProxy_AuthnCallback_do_fail_authnfailinfo_list (
 
 		callback = (EazelProxy_AuthnCallback) list_node->data;
 
-		if (CORBA_OBJECT_NIL != callback) {
+		if (CORBA_OBJECT_NIL != callback && ! callback_cancel_has_canceled (callback)) {
 			if (NULL == info) {
 				EazelProxy_AuthnCallback_failed (callback, user, &alt_fail_info, &ev);
 			} else {
@@ -1368,14 +1536,18 @@ impl_EazelProxy_AuthnCallback_do_fail_authnfailinfo (
 	if (CORBA_OBJECT_NIL != callback) {
 		if (NULL == info) {
 			log ("AuthnCallback::failed for user '%s' for reason '0' ''", user->user_name);
-			EazelProxy_AuthnCallback_failed (callback, user, &alt_fail_info, &ev);
+			if (!callback_cancel_has_canceled (callback) {
+				EazelProxy_AuthnCallback_failed (callback, user, &alt_fail_info, &ev);
+			}
 		} else {
 			log ("AuthnCallback::failed for user '%s' for reason '%d' '%s'", 
 				user->user_name,
 				info->code, 
 				info->http_result ? info->http_result : ""
 			);
-			EazelProxy_AuthnCallback_failed (callback, user, info, &ev);
+			if (!callback_cancel_has_canceled (callback) {
+				EazelProxy_AuthnCallback_failed (callback, user, info, &ev);
+			}
 		}
 	}
 
@@ -1464,7 +1636,7 @@ impl_EazelProxy_AuthnCallback_do_succeeded_list (
 		EazelProxy_AuthnCallback callback;
 
 		callback = (EazelProxy_AuthnCallback) list_node->data;
-		if (CORBA_OBJECT_NIL != callback) {
+		if (CORBA_OBJECT_NIL != callback && !callback_cancel_has_canceled (callback)) {
 			EazelProxy_AuthnCallback_succeeded (callback, user, &ev);
 		}
 	}
@@ -1607,10 +1779,9 @@ impl_EazelProxy_UserControl_prompt_authenticate_user (
 ) {
 	User *user;
 	User *user_listed;
-
+	
 	user = user_new_no_list_add (authninfo->username);
 
-	user->callback_list = g_list_prepend (user->callback_list, CORBA_Object_duplicate (callback, ev));
 	user->user.login_state = EazelProxy_AUTHENTICATING;
 	/* Note that this is a *lie* until user_activate is called*/
 	user->user.is_default = is_default;
@@ -1629,12 +1800,15 @@ impl_EazelProxy_UserControl_prompt_authenticate_user (
 
 	if (user_listed) {
 		if (EazelProxy_AUTHENTICATING == user_listed->user.login_state) {
+			/* This user is already waiting to be authenticated */
+			callback_cancel_add (callback);
 			user_listed->callback_list = 
 						g_list_prepend (
 							user_listed->callback_list, 
 							CORBA_Object_duplicate (callback, ev)
 						);
 		} else {
+			/* This user has already been authenticated */
 			EazelProxy_AuthnCallback_succeeded (callback, &(user_listed->user), ev);
 		}
 		user_free (user);
@@ -1643,6 +1817,9 @@ impl_EazelProxy_UserControl_prompt_authenticate_user (
 		/* The anonymous user gets logged in implicitly; it is not prompted for a password */
 		gl_user_list = g_list_prepend (gl_user_list, user);
 
+		callback_cancel_add (callback);
+		user->callback_list = g_list_prepend (user->callback_list, CORBA_Object_duplicate (callback, ev));
+	
 		my_authenticate_user (user, "", user_authenticate_cb);
 	} else {
 		CORBA_boolean has_password;
@@ -1652,6 +1829,10 @@ impl_EazelProxy_UserControl_prompt_authenticate_user (
 
 		gl_user_list = g_list_prepend (gl_user_list, user);
 
+		callback_cancel_add (callback);
+		user->prompter_authn_callback = CORBA_Object_duplicate (callback, ev);
+		user->callback_list = g_list_prepend (user->callback_list, user->prompter_authn_callback);
+
 		log ("Prompting user '%s' for authn", authninfo->username);
 
 		has_password = EazelProxy_UserPrompter_prompt_authenticate (prompter, &(user->user), EazelProxy_Initial, &new_authninfo, ev);
@@ -1715,6 +1896,7 @@ impl_EazelProxy_UserControl_authenticate_user (
 
 	user = user_new_no_list_add (authninfo->username);
 
+	callback_cancel_add (callback);
 	user->callback_list = g_list_prepend (user->callback_list, CORBA_Object_duplicate (callback, ev));
 	user->user.login_state = EazelProxy_AUTHENTICATING;
 	/* Note that this is a *lie* until user_activate is called*/
@@ -1759,7 +1941,6 @@ impl_EazelProxy_UserControl_set_new_user_password (
 
 	user = user_new_no_list_add (authninfo->username);
 
-	user->callback_list = g_list_prepend (user->callback_list, CORBA_Object_duplicate (callback, ev));
 	user->user.login_state = EazelProxy_AUTHENTICATING;
 	user_set_services_strings (user, authninfo);
 
@@ -1790,6 +1971,7 @@ impl_EazelProxy_UserControl_set_new_user_password (
 		authninfo_used->services_redirect_uri = CORBA_string_dup (user_listed->user.services_redirect_uri);
 		authninfo_used->services_login_path = CORBA_string_dup (user_listed->user.services_login_path); 
 
+		callback_cancel_add (callback);
 		set_user_password (user_listed, authninfo_used, new_password, callback_copy);
 	}
 
@@ -1797,6 +1979,15 @@ error:
 	return FALSE;
 }
 
+static void
+impl_EazelProxy_UserControl_cancel_async_operation (
+	PortableServer_Servant servant,
+	const EazelProxy_AuthnCallback callback,
+	CORBA_Environment * ev
+) {
+	callback_cancel_set_canceled (callback);
+}
+
 static CORBA_char *
 impl_EazelProxy_UserControl_get_authn_header_for_port (
 	PortableServer_Servant servant,
