@@ -1,4 +1,4 @@
-/* $Id: proxy.c,v 1.36 2001/01/18 08:55:31 ramiro Exp $
+/* $Id: proxy.c,v 1.38 2001/02/13 04:13:11 mfleming Exp $
  *
  * A trivial proxy server for relaying Eazel HTTP connections.  Sets up
  * initial config options, daemonizes if necessary, and then stays in a
@@ -78,9 +78,10 @@
 #define KEY_GCONF_USE_HTTP_PROXY (PATH_GCONF_GNOME_VFS ITEM_GCONF_USE_HTTP_PROXY)
 
 #define KEY_GCONF_DEFAULT_SSL_CERT "/apps/eazel-trilobite/ssl-cert"
+#define KEY_GCONF_DEFAULT_SSL_CERT_DIR "/apps/eazel-trilobite/ssl-cert-dir"
 #define KEY_GCONF_DEFAULT_DISABLE_SSL "/apps/eazel-trilobite/ssl-disable"
 
-#define EAZEL_DEFAULT_CERT "/nautilus/certs/eazel_dsa_cert.pem"
+#define EAZEL_DEFAULT_CERT_DIR "/nautilus/certs"
 
 void takesig (int sig);
 
@@ -109,9 +110,12 @@ struct stat_s stats;
 GMainLoop *l_main_loop = NULL;
 
 char *gl_opt_certfile = NULL;
+char *gl_opt_cert_dir = NULL;
+
 int gl_opt_no_ssl = 0;
 gboolean gl_gconf_ssl_disabled = FALSE;
 char *gl_gconf_ssl_cert = NULL;
+char *gl_gconf_ssl_cert_dir = NULL;
 
 
 static const struct poptOption popt_options[] = {
@@ -123,6 +127,7 @@ static const struct poptOption popt_options[] = {
 #ifdef HAVE_OPENSSL
 	{"no-ssl", '\0', POPT_ARG_NONE, &gl_opt_no_ssl, 0, "Do not make outbound connections use SSL", NULL },
 	{"ssl-cert", '\0', POPT_ARG_STRING, &gl_opt_certfile, 0, "SSL certificate to use to verify server ", "cert-file" },
+	{"ssl-cert-dir", '\0', POPT_ARG_STRING, &gl_opt_cert_dir, 0, "Directory containing SSL certs, all of which will be used.", "cert-directory" },
 #endif /* HAVE_OPENSSL */
 #endif /* SIMPLE_PROXY */
 	{"upstream-proxy", 'u', POPT_ARG_STRING, NULL, 'u', "use an upstream proxy for connections",
@@ -283,6 +288,7 @@ load_gconf_defaults (void)
 		{KEY_GCONF_HTTP_PROXY_HOST, GCONF_VALUE_STRING, {NULL}, watch_gconf_proxy_cb},
 		{KEY_GCONF_HTTP_PROXY_PORT, GCONF_VALUE_INT, {NULL}, watch_gconf_proxy_cb},
 		{KEY_GCONF_DEFAULT_SSL_CERT, GCONF_VALUE_STRING, {(gchar **)&gl_gconf_ssl_cert}, NULL},
+		{KEY_GCONF_DEFAULT_SSL_CERT_DIR, GCONF_VALUE_STRING, {(gchar **)&gl_gconf_ssl_cert_dir}, NULL},
 		{KEY_GCONF_DEFAULT_DISABLE_SSL, GCONF_VALUE_BOOL, {(gchar **) /*gboolean*/&gl_gconf_ssl_disabled}, NULL}
 	};
 
@@ -470,12 +476,16 @@ int main (int argc,
 	if (gl_opt_certfile) {
 		gl_opt_certfile = g_strdup (gl_opt_certfile);
 	}
+
+	if (gl_opt_cert_dir) {
+		gl_opt_certfile = g_strdup (gl_opt_cert_dir);
+	}
+
 #endif
 
 	poptFreeContext (popt_context);
 	
 	if (godaemon) {
-		fprintf (stderr, "eazel-proxy launching into the background...\n\n");
 		make_daemon ();
 	}
 
@@ -502,12 +512,18 @@ int main (int argc,
 			if (gl_opt_certfile) {
 				log ("Enabling SSL; using certfile %s", gl_opt_certfile);
 				eazel_init_ssl (NULL, gl_opt_certfile);
+			} else if (gl_opt_cert_dir) {
+				log ("Enabling SSL; using cert directory %s", gl_opt_cert_dir);
+				eazel_init_ssl (gl_opt_cert_dir, NULL);
 			} else if (gl_gconf_ssl_cert) {
-				log ("Enabling SSL; using certfile %s", gl_opt_certfile);
+				log ("Enabling SSL; using certfile %s", gl_gconf_ssl_cert);
+				eazel_init_ssl (NULL, gl_gconf_ssl_cert);
+			} else if (gl_gconf_ssl_cert_dir) {
+				log ("Enabling SSL; using cert directory %s", gl_gconf_ssl_cert);
 				eazel_init_ssl (NULL, gl_gconf_ssl_cert);
 			} else {
-				log ("Enabling SSL; using certfile %s", DATADIR EAZEL_DEFAULT_CERT);
-				eazel_init_ssl (NULL, DATADIR EAZEL_DEFAULT_CERT);
+				log ("Enabling SSL; using cert directory %s", DATADIR EAZEL_DEFAULT_CERT_DIR);
+				eazel_init_ssl (DATADIR EAZEL_DEFAULT_CERT_DIR, NULL );
 			}
 		}
 #endif
