@@ -12,12 +12,14 @@ typedef struct {
 	char *upstream_host;
 	int upstream_port;
 	gboolean specified_upstream_host;
+	char *local_host;
 	int local_port;
 	gboolean append_log;
 	gboolean use_stderr;
 	gboolean use_ssl;
 	gboolean use_oaf;
 	gboolean quit;
+	gboolean check_uid;
 #ifndef NO_DEBUG_MIRRORING
 	char *mirror_log_dir;
 #endif
