@@ -1,4 +1,4 @@
-/* $Id: request.c,v 1.42 2001/01/16 21:30:08 mfleming Exp $
+/* $Id: request.c,v 1.43 2001/02/23 12:23:35 seth Exp $
  *
  * Handles incoming request callbacks, buffers headers, and passes
  * the request on to the upstream server.
@@ -680,8 +680,8 @@ proxy_handle_connect_method (Connection *conn, Socket *client_sock)
 		char *in_mirror_filename, *out_mirror_filename;
 		FILE *in_mirror, *out_mirror;
 
-		in_mirror_filename = g_strdup_printf ("%s/ep-%d-%d-fm-server.log", config.mirror_log_dir, getpid(), conn->id);
-		out_mirror_filename = g_strdup_printf ("%s/ep-%d-%d-to-server.log", config.mirror_log_dir, getpid(), conn->id);
+		in_mirror_filename = g_strdup_printf ("%s/ep-%ld-%d-fm-server.log", config.mirror_log_dir, (long int) getpid(), conn->id);
+		out_mirror_filename = g_strdup_printf ("%s/ep-%ld-%d-to-server.log", config.mirror_log_dir, (long int) getpid(), conn->id);
 		in_mirror = fopen (in_mirror_filename, "w");
 		out_mirror = fopen (out_mirror_filename, "w");
 		socket_set_mirrors (conn->server, out_mirror, in_mirror);
@@ -785,8 +785,8 @@ proxy_do_connect(Socket *client_sock)
 		if (config.mirror_log_dir) {
 			char *in_mirror_filename, *out_mirror_filename;
 
-			in_mirror_filename = g_strdup_printf ("%s/ep-%d-%d-fm-server.log", config.mirror_log_dir, getpid(), conn->id);
-			out_mirror_filename = g_strdup_printf ("%s/ep-%d-%d-to-server.log", config.mirror_log_dir, getpid(), conn->id);
+			in_mirror_filename = g_strdup_printf ("%s/ep-%ld-%d-fm-server.log", config.mirror_log_dir, (long int) getpid(), conn->id);
+			out_mirror_filename = g_strdup_printf ("%s/ep-%ld-%d-to-server.log", config.mirror_log_dir, (long int) getpid(), conn->id);
 			in_mirror = fopen (in_mirror_filename, "w");
 			out_mirror = fopen (out_mirror_filename, "w");
 			g_free (in_mirror_filename);
@@ -1074,8 +1074,8 @@ proxy_new_connection(Socket *client_sock, gpointer user_data, const char *host,
 		char *in_mirror_filename;
 		char *out_mirror_filename;
 
-		in_mirror_filename = g_strdup_printf ("%s/ep-%d-%d-fm-client.log", config.mirror_log_dir, getpid(), conn->id);
-		out_mirror_filename = g_strdup_printf ("%s/ep-%d-%d-to-client.log", config.mirror_log_dir, getpid(), conn->id);
+		in_mirror_filename = g_strdup_printf ("%s/ep-%ld-%d-fm-client.log", config.mirror_log_dir, (long int) getpid(), conn->id);
+		out_mirror_filename = g_strdup_printf ("%s/ep-%ld-%d-to-client.log", config.mirror_log_dir, (long int) getpid(), conn->id);
 		in_mirror = fopen (in_mirror_filename, "w");
 		out_mirror = fopen (out_mirror_filename, "w");
 		socket_set_mirrors (client_sock, out_mirror, in_mirror);
