@@ -1,4 +1,4 @@
-/* $Id: sock-ssl.c,v 1.8 2000/12/06 01:21:32 mfleming Exp $
+/* $Id: sock-ssl.c,v 1.10 2001/02/18 04:51:25 mfleming Exp $
  *
  * Handles the ugliness of using the openSSL library.
  *
@@ -26,15 +26,6 @@
 /* only build this file if openssl is installed */
 #ifdef HAVE_OPENSSL
 
-/* This is stuff to disable potentially problematic crypto */
-#define NO_MD2 
-#define NO_MD4 
-#define NO_RC4 
-#define NO_RC5 
-#define NO_MDC2 
-#define NO_IDEA 
-#define NO_RSA 
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -42,6 +33,9 @@
 #include <openssl/err.h>
 #include <glib.h>
 
+#include <sys/types.h>
+#include <dirent.h>
+
 #include "log.h"
 #include "sock-ssl.h"
 
@@ -68,9 +62,27 @@ verify_callback(int state, X509_STORE_CTX *x509_context)
 
 	err = X509_STORE_CTX_get_error (x509_context);
 	if (! state) {
+#if 0
+		/* I just have this here so I can gobble up certs */
+		static int counter = 0;
+		char * filename;
+		FILE * file;
+
+		filename = g_strdup_printf ("/tmp/cert-%d.pem", counter);
+
+		file = fopen (filename, "a");
+						
+		PEM_write_X509 (file, x509_context->current_cert);
+		log ("Wrote cert '%s'", filename);
+
+		fclose (file);
+		g_free (filename);
+#endif
+
 		X509_NAME_oneline (X509_get_subject_name (x509_context->current_cert), name, sizeof(name));
 		/* Remote site specified a certificate, but it's not correct */
 		log ("ssl cert error: %s: %s", X509_verify_cert_error_string(x509_context->error), name);
+
 		/* reject */
 		return 0;
 	}
@@ -99,6 +111,54 @@ ssl_log_error(const char *prefix)
 	}
 }
 
+static gboolean
+string_ends_in (const char *string, const char *suffix)
+{
+	size_t string_len, suffix_len;
+
+	string_len = strlen (string);
+	suffix_len = strlen (suffix);
+
+	if (suffix_len > string_len) {
+		return FALSE;
+	} else {
+		return 0 == strcmp (string + strlen (string) - strlen (suffix), suffix);
+	}
+}
+
+static void
+load_all_certs_in_directory (const char *cert_directory)
+{
+	DIR *dir_handle;
+	struct dirent *entry;
+	char *full_filename;
+
+	if (NULL == cert_directory) {
+		return;
+	}
+
+
+	dir_handle = opendir (cert_directory);
+
+	if (NULL == dir_handle) {
+		return;
+	}
+
+	log ("Loading all certs in %s", cert_directory);
+
+	while (NULL != (entry = readdir (dir_handle))) {
+		if (string_ends_in (entry->d_name, ".pem")) {
+			full_filename = g_strconcat (cert_directory, "/", entry->d_name, NULL);
+			if (SSL_CTX_load_verify_locations (ssl_context, full_filename, NULL)) {
+				log ("PEM certificates in '%s' loaded", entry->d_name);
+			} else {
+				log ("ERROR loading PEM certificates in '%s'", entry->d_name);
+			}
+			g_free (full_filename);
+			full_filename = NULL;
+		}
+	}
+}
 
 /* initialize the openssl library and set up some callbacks and paths.
  *     cert_directory: a directory where CA certs can be found
@@ -135,9 +195,15 @@ eazel_init_ssl(char *cert_directory, char *cert_file)
 		exit (1);
 	}
 
-	if (! SSL_CTX_load_verify_locations (ssl_context, cert_file, cert_directory)) {
-		ssl_log_error ("FATAL: openssl cert loading failed: ");
-		exit(1);
+	if (NULL != cert_file) {
+		if (! SSL_CTX_load_verify_locations (ssl_context, cert_file, NULL)) {
+			ssl_log_error ("FATAL: openssl cert loading failed: ");
+			exit(1);
+		}
+	}
+
+	if (NULL != cert_directory) {
+		load_all_certs_in_directory (cert_directory);
 	}
 
 #ifdef VERIFY_CERTS
@@ -145,59 +211,58 @@ eazel_init_ssl(char *cert_directory, char *cert_file)
 #endif
 }
 
-
 /* turn off SSL ciphers that have patent issues */
 static int
 socket_turn_off_evil_ciphers(SSL *ssl_handle)
 {
-	STACK_OF(SSL_CIPHER) *stack;
-	char *rule = NULL, *newbuf;
-	const char *cipher_name;
-	int i;
-
-	stack = SSL_get_ciphers (ssl_handle);
-	for (i = 0; i < sk_SSL_CIPHER_num (stack); i++) {
-		cipher_name = SSL_CIPHER_get_name (sk_SSL_CIPHER_value (stack, i));
-		/* RSA: patented till around October 2000 */
-		/* IDEA & RC5: patented till longer */
-		/* RC4: questionable (leaked trade secret?) */
-		if ((strstr (cipher_name, "RSA") != NULL) ||
-		    (strstr (cipher_name, "IDEA") != NULL) ||
-		    (strstr (cipher_name, "RC4") != NULL) ||
-		    (strstr (cipher_name, "RC5") != NULL)) {
-			if (rule) {
-				newbuf = g_strconcat (rule, ":-", cipher_name, NULL);
-				g_free(rule);
-				rule = newbuf;
-			} else {
-				rule = g_strconcat ("DEFAULT:-", cipher_name, NULL);
-			}
-		}
-	}
-	/* stack is not allocated, don't free it. */
-
-#ifdef DEBUG_CIPHER_LIST
-	log ("SSL cipher list rule: '%s'", rule ? rule : "(none)");
-#endif
-	if (rule && (! SSL_set_cipher_list (ssl_handle, rule))) {
-		g_free (rule);
-		rule = NULL;
-		ssl_log_error ("error: ssl_set_cipher_list: ");
-		return SOCKET_SSL_ERROR_OPENSSL;
-	}
-	g_free (rule);
-	rule = NULL;
-
-#ifdef DEBUG_CIPHER_LIST
-	/* debug list */
-	stack = SSL_get_ciphers (ssl_handle);
-	log ("SSL cipher list: (%d entries)", sk_SSL_CIPHER_num (stack));
-	for (i = 0; i < sk_SSL_CIPHER_num (stack); i++) {
-		log ("stack %d: %s", i, SSL_CIPHER_get_name (sk_SSL_CIPHER_value (stack, i)));
-	}
-#endif
-
-	return 0;
+ 	STACK_OF(SSL_CIPHER) *stack;
+ 	char *rule = NULL, *newbuf;
+ 	const char *cipher_name;
+ 	int i;
+ 
+ 	stack = SSL_get_ciphers (ssl_handle);
+ 	for (i = 0; i < sk_SSL_CIPHER_num (stack); i++) {
+ 		cipher_name = SSL_CIPHER_get_name (sk_SSL_CIPHER_value (stack, i));
+ 		/* RSA: patented till around October 2000 */
+ 		/* IDEA & RC5: patented till longer */
+ 		/* RC4: questionable (leaked trade secret?) */
+ 		if ((strstr (cipher_name, "RSA") != NULL) ||
+ 		    (strstr (cipher_name, "IDEA") != NULL) ||
+ 		    (strstr (cipher_name, "RC4") != NULL) ||
+ 		    (strstr (cipher_name, "RC5") != NULL)) {
+ 			if (rule) {
+ 				newbuf = g_strconcat (rule, ":-", cipher_name, NULL);
+ 				g_free(rule);
+ 				rule = newbuf;
+ 			} else {
+ 				rule = g_strconcat ("DEFAULT:-", cipher_name, NULL);
+ 			}
+ 		}
+ 	}
+ 	/* stack is not allocated, don't free it. */
+ 
+ #ifdef DEBUG_CIPHER_LIST
+ 	log ("SSL cipher list rule: '%s'", rule ? rule : "(none)");
+ #endif
+ 	if (rule && (! SSL_set_cipher_list (ssl_handle, rule))) {
+ 		g_free (rule);
+ 		rule = NULL;
+ 		ssl_log_error ("error: ssl_set_cipher_list: ");
+ 		return SOCKET_SSL_ERROR_OPENSSL;
+ 	}
+ 	g_free (rule);
+ 	rule = NULL;
+ 
+ #ifdef DEBUG_CIPHER_LIST
+ 	/* debug list */
+ 	stack = SSL_get_ciphers (ssl_handle);
+ 	log ("SSL cipher list: (%d entries)", sk_SSL_CIPHER_num (stack));
+ 	for (i = 0; i < sk_SSL_CIPHER_num (stack); i++) {
+ 		log ("stack %d: %s", i, SSL_CIPHER_get_name (sk_SSL_CIPHER_value (stack, i)));
+ 	}
+ #endif
+ 
+ 	return 0;
 }
 
 
@@ -231,11 +296,8 @@ ssl_begin_ssl(int fd, int *error)
 		goto bad;
 	}
 
-	err = socket_turn_off_evil_ciphers (ssl_handle);
-	if (err) {
-		*error = err;
-		goto bad;
-	}
+	/* SLAM for bug 6733: Remove RSA so that we use the DSA cert for now */
+	socket_turn_off_evil_ciphers (ssl_handle);
 
 	err = SSL_connect (ssl_handle);
 	if (err == 0) {
