@@ -1,5 +1,5 @@
-/* $Id: eazel-proxy-util.c,v 1.40 2001/01/19 00:08:09 robey Exp $
- * 
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: 8; c-basic-offset: 8 -*- */
+/* 
  * eazel-proxy-util:
  * 
  * A client an tool for "eazel-proxy", aka ammonite.
@@ -33,24 +33,27 @@
 #include <libammonite.h>
 #include <libammonite-gtk.h>
 
-#include <stdio.h>
-#include <orb/orbit.h>
-#include <liboaf/liboaf.h>
-#include <popt.h>
-#include <unistd.h>
-#include <signal.h>
-#include <glib.h>
-#include <stdlib.h>
-#include <time.h>
 #include <gnome.h>
 #include <libgnomeui/libgnomeui.h> 
+#include <liboaf/liboaf.h>
 #include <gconf/gconf.h>
+#include <orb/orbit.h>
+#include <popt.h>
+#include <glib.h>
+#include <unistd.h>
+#include <signal.h>
+#include <stdlib.h>
+#include <time.h>
+#include <stdio.h>
 
 /* If authentication fails--for whatever reason--ignore subsequent requests for a few seconds
  * so we don't get redundent login boxes from the same operation
  */
 #define IGNORE_LOGIN_AFTER_FAIL_SECONDS 2
 
+/* for debugging */
+#undef ALT_PROMPT_CODE
+
 #ifdef MKF_DEBUG
 #define DEBUG_MSG(x) fprintf x;
 #else
@@ -60,28 +63,25 @@
 EazelProxy_UserControl gl_user_control;
 PortableServer_POA gl_poa;
 
-char * opt_login	= NULL;
+const char *opt_login  = NULL;
+const char *opt_change_password_username  = NULL;
+int opt_login_dialog	= 0;
 int opt_login_prompt	= 0;
 int opt_status		= 0;
 int opt_follow		= 0;
-char * opt_password	= NULL;
-char * opt_logout	= NULL;
+char *opt_password	= NULL;
+char *opt_logout	= NULL;
 int opt_translate	= 0;
 int opt_help		= 0;
-char * opt_target_path  = NULL;
-char * opt_login_path	= NULL;
-#if 0
-int opt_dialog		= 0;
-#endif
+char *opt_target_path   = NULL;
+char *opt_login_path	= NULL;
 int opt_chpw		= 0;
 #ifdef ENABLE_REAUTHN
 int opt_reauthn_listen	= 0;
 #endif /* ENABLE_REAUTHN */
 
-static const struct poptOption popt_options[] = {
-#if 0
-	{"dialog", 'd', POPT_ARG_NONE, &opt_dialog, 0, "Prompt for login w/ dialog box", NULL},
-#endif
+static const struct poptOption full_popt_options[] = {
+	{"dialog", 'd', POPT_ARG_NONE, &opt_login_dialog, 0, "Log in default eazel service user, prompting w/ dialog box if necessary", NULL},
 	{"login", 'l', POPT_ARG_STRING, &opt_login, 0, "Log in eazel service user", "<username>"},
 	{"login-prompt", 'P', POPT_ARG_NONE, &opt_login_prompt, 0, "Log in default eazel service user, prompting if necessary", NULL},
 	{"status", 's', POPT_ARG_NONE, &opt_status, 0, "Print status", NULL},
@@ -101,20 +101,47 @@ static const struct poptOption popt_options[] = {
 	{NULL, '\0', 0, NULL, 0, NULL}
 };
 
+
+static const struct poptOption login_popt_options[] = {
+	{"password", 'p', POPT_ARG_STRING, &opt_password, 0, "Password to use for login", "<passwd>"},
+	{"target-path", 't', POPT_ARG_STRING, &opt_target_path, 0, "Target path to redirect to", "<url>"},
+	{"login-path", 'L', POPT_ARG_STRING, &opt_login_path, 0, "Path to login cgi (relative to target path)",
+		"<path>"},
+	POPT_AUTOHELP
+	{NULL, 'h', POPT_ARG_NONE | POPT_ARGFLAG_DOC_HIDDEN, &opt_help, 0, NULL, NULL},
+	{NULL, '\0', 0, NULL, 0, NULL}
+};
+
+
+static const struct poptOption logout_popt_options[] = {
+	{"logout", 'u', POPT_ARG_STRING, &opt_logout, 0, "Logout user on port", "<port number>"},
+	POPT_AUTOHELP
+	{NULL, 'h', POPT_ARG_NONE | POPT_ARGFLAG_DOC_HIDDEN, &opt_help, 0, NULL, NULL},
+	{NULL, '\0', 0, NULL, 0, NULL}
+};
+
+static const struct poptOption change_password_popt_options[] = {
+	POPT_AUTOHELP
+	{NULL, 'h', POPT_ARG_NONE | POPT_ARGFLAG_DOC_HIDDEN, &opt_help, 0, NULL, NULL},
+	{NULL, '\0', 0, NULL, 0, NULL}
+};
+
+
+
 static void do_login (EazelProxy_UserControl cntl, const char *login, const char *passwd, const char *target_path,
 		      const char *login_path);
-static void do_login_prompt (EazelProxy_UserControl cntl, const char *target_path, const char *login_path);
+static void do_login_dialog (EazelProxy_UserControl cntl, const char *target_path, const char *login_path);
 static void do_status (EazelProxy_UserControl cntl);
 static void do_follow (EazelProxy_UserControl cntl);
 static void do_logout (EazelProxy_UserControl cntl, const char *username);
 static void do_translate (EazelProxy_UserControl cntl) ;
-static void do_change_password (EazelProxy_UserControl cntl);
+static void do_change_password (EazelProxy_UserControl cntl, const char *username);
 
 EazelProxy_User *
 eazel_proxy_util_do_login (EazelProxy_UserControl cntl, const char *login, const char *passwd, const char *target_path,
 	  const char *login_path, /*OUT*/ CORBA_long *p_fail_code);
 
-#if 0
+#ifdef ALT_PROMPT_CODE
 static EazelProxy_User *
 eazel_proxy_util_do_prompt_login (
 	EazelProxy_UserControl cntl, 
@@ -123,7 +150,7 @@ eazel_proxy_util_do_prompt_login (
 	const char *login_path,
 	/* OUT */ CORBA_long *p_fail_code
 );
-#endif /* 0 */
+#endif
 
 EazelProxy_User *
 eazel_proxy_util_change_password (
@@ -134,28 +161,27 @@ eazel_proxy_util_change_password (
         int *fail_code
 );
 
+static void print_bad_arguments_message (const char *program_name);
+static void parse_arguments             (int argc,
+					 char **argv,
+					 const struct poptOption *popt_options,
+					 poptContext *popt_context);
+
 
-static void
-illegal_options (poptContext context)
-{
-	
-	fprintf (stderr, "Illegal combination of options specified\n");
 
-	poptPrintUsage (context , stderr, 0);
-}
 
 int main (int argc, char **argv)
 {
 	CORBA_Environment ev;
 	CORBA_ORB orb;
-	poptContext popt_context;
-	int rc;
 	GError *gerror = NULL;
 	char * fake_argv[] = {"eazel-proxy-util", NULL};
+	const char **args;
+	poptContext popt_context;
 
 	CORBA_exception_init (&ev);
 
-	DEBUG_MSG ((stderr,"%lu: In eazel-proxy-util ppid %lu\n", (unsigned long) getpid(), (unsigned long)getppid()));
+	DEBUG_MSG ((stderr,"%lu: In eazel-proxy-util ppid %lu\n", (unsigned long) getpid (), (unsigned long) getppid ()));
 
 	orb = oaf_init (argc, argv);
 
@@ -165,8 +191,8 @@ int main (int argc, char **argv)
 	}
 
 	/* Get the POA for further object activation */
-	gl_poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(orb, "RootPOA", &ev);
-	PortableServer_POAManager_activate(PortableServer_POA__get_the_POAManager(gl_poa, &ev), &ev);
+	gl_poa = (PortableServer_POA) CORBA_ORB_resolve_initial_references (orb, "RootPOA", &ev);
+	PortableServer_POAManager_activate (PortableServer_POA__get_the_POAManager (gl_poa, &ev), &ev);
 
 	/* init gconf, which is needed by ammonite */
 	if (!gconf_init (1, fake_argv, &gerror)) {
@@ -174,48 +200,64 @@ int main (int argc, char **argv)
 		exit (-1);
 	}
 
-	if (! ammonite_init(gl_poa) ) {
+	if (!ammonite_init (gl_poa)) {
 		fprintf (stderr, "Error during ammonite_init()\n");
 		exit (-1);
 	}
-	gl_user_control = ammonite_get_user_control();
+	gl_user_control = ammonite_get_user_control ();
 
-	popt_context = poptGetContext("eazel-proxy-control", argc, (const char **)argv, popt_options, 0);
 
-	if (-1 != (rc = poptGetNextOpt (popt_context) ) ) {
-		exit (-1);
+	if (strcmp (argv[0], "eazel-login") == 0) {
+		parse_arguments (argc, argv,
+				 login_popt_options, &popt_context);
+
+		args = poptGetArgs (popt_context);
+
+		if (args != NULL) {
+  		        opt_login = args[0];
+		} else {
+		        opt_login_prompt = 1;
+		}
+	} else if (strcmp (argv[0], "eazel-logout") == 0) {
+		parse_arguments (argc, argv,
+				 logout_popt_options, &popt_context);
+	} else if (strcmp (argv[0], "eazel-passwd") == 0) {
+		parse_arguments (argc, argv,
+				 change_password_popt_options, &popt_context);
+
+		args = poptGetArgs (popt_context);
+
+		if (args != NULL) {
+			opt_change_password_username = args[0];
+		}
+
+		opt_chpw = 1;
+	} else {
+		parse_arguments (argc, argv,
+				 full_popt_options, &popt_context);
 	}
 
-	if (opt_help) {
-		poptPrintUsage (popt_context, stderr, 0);
-		exit (0);
-	}
-
-	/* mfleming -- this is no longer an accurate representation of valid combinations */
+	/* FIXME -- this is no longer an accurate representation of valid combinations */
 	if ( opt_login && ( opt_status || opt_follow || opt_logout || opt_translate ) ) {
-		illegal_options(popt_context);
-		exit (-1);
-	} else if ( opt_status && ( opt_login || opt_follow || opt_password || opt_logout || opt_translate) ) {
-		illegal_options(popt_context);
-		exit (-1);
-	} else if ( opt_follow && ( opt_login || opt_status || opt_password || opt_logout || opt_translate) ) {
-		illegal_options(popt_context);
-		exit (-1);
-	} else if ( opt_password &&  !opt_login ) {
-		illegal_options(popt_context);
-		exit (-1);
-	} else if ( opt_logout && ( opt_login || opt_status || opt_password || opt_follow || opt_translate) ) {
-		illegal_options(popt_context);
-		exit (-1);
-	} else if ( opt_translate && ( opt_login || opt_status || opt_password || opt_follow || opt_logout) ) {
-		illegal_options(popt_context);
-		exit (-1);
+		print_bad_arguments_message (argv[0]);
+	} else if (opt_status && (opt_login || opt_follow || opt_password || opt_logout || opt_translate)) {
+		print_bad_arguments_message (argv[0]);
+	} else if (opt_follow && (opt_login || opt_status || opt_password || opt_logout || opt_translate)) {
+		print_bad_arguments_message (argv[0]);
+	} else if (opt_password &&  !opt_login) {
+		print_bad_arguments_message (argv[0]);
+	} else if (opt_logout && (opt_login || opt_status || opt_password || opt_follow || opt_translate)) {
+		print_bad_arguments_message (argv[0]);
+	} else if (opt_translate && (opt_login || opt_status || opt_password || opt_follow || opt_logout)) {
+		print_bad_arguments_message (argv[0]);
 	}
 
 	if (opt_login) {
 		do_login (gl_user_control, opt_login, opt_password, opt_target_path, opt_login_path);
 	} else if (opt_login_prompt) {
-		do_login_prompt (gl_user_control, opt_target_path, opt_login_path);
+		do_login (gl_user_control, NULL, opt_password, opt_target_path, opt_login_path);
+	} else if (opt_login_dialog) {
+		do_login_dialog (gl_user_control, opt_target_path, opt_login_path);
 	} else if (opt_status) {
 		do_status (gl_user_control);
 	} else if (opt_follow) {
@@ -224,12 +266,8 @@ int main (int argc, char **argv)
 		do_logout (gl_user_control, opt_logout);
 	} else if (opt_translate) {
 		do_translate (gl_user_control) ;
-#if 0
-	} else if (opt_dialog) {
-		eazel_proxy_util_do_dialog ();
-#endif /* 0 */
 	} else if (opt_chpw) {
-		do_change_password (gl_user_control);
+		do_change_password (gl_user_control, opt_change_password_username);
 #ifdef ENABLE_REAUTHN
 	} else if (opt_reauthn_listen) {
 		eazel_proxy_util_do_reauthn_listen (gl_user_control);
@@ -359,42 +397,51 @@ authn_cb_failed (const EazelProxy_User *user, const EazelProxy_AuthnFailInfo *in
 }
 
 static void
-do_login (EazelProxy_UserControl cntl, const char *login, const char *passwd, const char *target_path,
+do_login (EazelProxy_UserControl cntl, 
+	  const char *login, 
+	  const char *passwd, 
+	  const char *target_path,
 	  const char *login_path)
 {
-	char * read_passwd = NULL;
+        char *prompt;
+        char buf[256];
+        char *read_login = NULL;
+	char *read_passwd = NULL;
 	EazelProxy_User *user;
 	CORBA_long fail_code;
 
-	if (NULL == passwd ) {
-		char *prompt;
-
-		prompt = g_strdup_printf ("Password for user '%s':", login);
-		read_passwd = getpass (prompt);
-		g_free (prompt);
+	if (login == NULL) {
+	        printf ("Eazel Services username: ");
+	        login = g_strchomp (fgets (buf, 256, stdin));
 	}
 
-	user = eazel_proxy_util_do_login (cntl, login,  
-				NULL == passwd ? read_passwd : passwd, 
-				target_path, login_path, &fail_code
-	       );
+	if (passwd == NULL) { 
+	         prompt = g_strdup_printf ("Password for user '%s': ", login);
+	         passwd = getpass (prompt);
+	         g_free (prompt);
+	}
+
+	user = eazel_proxy_util_do_login (cntl, login, passwd,
+					  target_path, login_path, &fail_code);
 
 	g_free (read_passwd);
+	g_free (read_login);
 
-	if ( NULL != user ) {
+	if (user != NULL) {
 		printf ("Login succeeded.\n");
 		CORBA_free (user);
 	} else {
 		printf ("Login FAILED '%s' (%d).\n", ammonite_fail_code_to_string (fail_code), fail_code);
 	}
+
 }
 
+
 static void
-do_login_prompt (
-	EazelProxy_UserControl cntl, 
-	const char *target_path, 
-	const char *login_path
-) {
+do_login_dialog (EazelProxy_UserControl cntl, 
+		 const char *target_path, 
+		 const char *login_path)
+{
 	EazelProxy_User *user;
 	gboolean gnome_inited;
 	CORBA_long fail_code;
@@ -406,8 +453,17 @@ do_login_prompt (
 		exit (-1);
 	}
 
+/* for debugging */
+#ifdef ALT_PROMPT_CODE
+	user = eazel_proxy_util_do_prompt_login (
+		ammonite_get_user_control(),
+		NULL, 
+		target_path, 
+		login_path,
+		&fail_code);
+#else
 	user = ammonite_do_prompt_login (NULL, target_path, login_path, TRUE, &fail_code);
-
+#endif
 	if ( NULL != user ) {
 		printf ("Login succeeded for user '%s' on port %d.\n", 
 			user->user_name, 
@@ -419,6 +475,7 @@ do_login_prompt (
 	}
 }
 
+
 EazelProxy_User *
 eazel_proxy_util_do_login (EazelProxy_UserControl cntl, const char *login, const char *passwd, const char *target_path,
 	  const char *login_path, /*OUT*/ CORBA_long *p_fail_code)
@@ -478,7 +535,8 @@ eazel_proxy_util_do_login (EazelProxy_UserControl cntl, const char *login, const
 	return user;
 }
 
-#if 0
+/* for debugging*/
+#ifdef ALT_PROMPT_CODE
 
 static CORBA_boolean
 prompter_cb_prompt_authenticate (
@@ -497,7 +555,7 @@ prompter_cb_prompt_authenticate (
 	if ( 	EazelProxy_InitialFail == kind
 		|| EazelProxy_ReauthnFail == kind
 	) {
-		eazel_proxy_util_do_error_dialog();
+		ammonite_do_authn_error_dialog();
 
 		*authninfo = EazelProxy_AuthnInfo__alloc();
 
@@ -510,9 +568,9 @@ prompter_cb_prompt_authenticate (
 		 * the user to change it
 		 */
 		if ( '\0' == user->user_name[0] ) {
-			ret = eazel_proxy_util_do_prompt_dialog (NULL, NULL, kind == EazelProxy_Initial, &username_glib, &password_glib);
+			ret = ammonite_do_prompt_dialog (NULL, NULL, kind == EazelProxy_Initial, &username_glib, &password_glib);
 		} else {
-			ret = eazel_proxy_util_do_prompt_dialog (user->user_name, NULL, kind == EazelProxy_Initial, NULL, &password_glib);
+			ret = ammonite_do_prompt_dialog (user->user_name, NULL, kind == EazelProxy_Initial, NULL, &password_glib);
 		}
 
 		/* Clean up events after the dialog */
@@ -542,6 +600,18 @@ prompter_cb_prompt_authenticate (
 	return ret;
 }
 
+volatile gboolean gl_cancel = FALSE;
+
+static void 
+take_signal (int sig)
+{
+	switch (sig) {
+	case SIGUSR1:
+		gl_cancel = TRUE;
+		break;
+	}
+}
+
 static EazelProxy_User *
 eazel_proxy_util_do_prompt_login (
 	EazelProxy_UserControl cntl,
@@ -567,6 +637,9 @@ eazel_proxy_util_do_prompt_login (
 
 	CORBA_exception_init (&ev);
 
+	/* This is only really intended for cancellation testing */
+	signal (SIGUSR1, take_signal);
+
 	userprompter = ammonite_userprompter_wrapper_new (gl_poa, &up_cb_funcs, (gpointer)&state);
 	authn_callback = ammonite_auth_callback_wrapper_new (gl_poa, &authn_cb_funcs, (gpointer)&state);
 
@@ -603,6 +676,12 @@ eazel_proxy_util_do_prompt_login (
 		fprintf (stderr, "Exception during prompt_authenticate_user\n");
 	} else {
 		while ( ! state.done && ! CORBA_Object_non_existent (cntl, &ev) ) {
+			if (gl_cancel) {
+				DEBUG_MSG ((stderr, "Received SIGTERM, sending cancel\n"));
+				EazelProxy_UserControl_cancel_async_operation (cntl, authn_callback, &ev);
+				signal (SIGUSR1, SIG_IGN);
+				gl_cancel = FALSE;
+			}
 			g_main_iteration(TRUE);
 		}
 
@@ -620,12 +699,12 @@ eazel_proxy_util_do_prompt_login (
 		DEBUG_MSG ((stderr, "%lu: Response iteration complete, success=%d\n", (unsigned long) getpid(), state.success));
 	}
 
-	if ( ! state.success 
+	if ( ! state.success && ! gl_cancel
 		  && ( EAZELPROXY_AUTHN_FAIL_NETWORK == state.code
 		  		|| EAZELPROXY_AUTHN_FAIL_SERVER == state.code
 		  )
 	) {
-		eazel_proxy_util_do_network_error_dialog();
+		ammonite_do_network_error_dialog();
 	}
 
 	ammonite_auth_callback_wrapper_free (gl_poa, authn_callback);
@@ -636,7 +715,7 @@ eazel_proxy_util_do_prompt_login (
 	return user;
 
 }
-#endif /*0*/
+#endif /* ALT_PROMPT_CODE */
 
 static const char *
 LoginState_to_string (EazelProxy_LoginState state )
@@ -1070,9 +1149,9 @@ eazel_proxy_util_change_password (
 
 #define USERNAME_SIZE 256
 static void
-do_change_password (EazelProxy_UserControl cntl)
+do_change_password (EazelProxy_UserControl cntl, const char *username)
 {
-	char * username = NULL;
+	char * read_username = NULL;
 	char * orig_passwd = NULL;
 	char * new_passwd0 = NULL;
 	char * new_passwd1 = NULL;
@@ -1080,16 +1159,19 @@ do_change_password (EazelProxy_UserControl cntl)
         int fail_code = 0;
 	EazelProxy_User *user;
 
-	username = g_malloc (USERNAME_SIZE * sizeof (char));
-	printf ("Username: ");
-	username = g_strchomp (fgets (username, USERNAME_SIZE, stdin));
+	if (username == NULL) {
+		read_username = g_malloc (USERNAME_SIZE * sizeof (char));
+		printf ("Username: ");
+		read_username = g_strchomp (fgets (read_username, USERNAME_SIZE, stdin));
+		username = read_username;
+	}
 
-	prompt = g_strdup_printf ("Original password for user '%s':", username);
+	prompt = g_strdup_printf ("Original password for user '%s': ", username);
 	orig_passwd = g_strdup (getpass (prompt));
 	g_free (prompt);
 
-	new_passwd0 = g_strdup (getpass ("New Password:"));
-	new_passwd1 = g_strdup (getpass ("Please retype new Password:"));
+	new_passwd0 = g_strdup (getpass ("New Password: "));
+	new_passwd1 = g_strdup (getpass ("Please retype new Password: "));
 
 	if (0 != strcmp (new_passwd0, new_passwd1)) {
 		printf ("New password entries do not match.  Password not changed.\n");
@@ -1100,7 +1182,7 @@ do_change_password (EazelProxy_UserControl cntl)
                                                  orig_passwd, new_passwd0, &fail_code
                                                  );
 
-	if ( NULL != user ) {
+	if (NULL != user) {
 		printf ("Change password succeeded.\n");
 		CORBA_free (user);
 	} else {
@@ -1127,3 +1209,31 @@ do_change_password (EazelProxy_UserControl cntl)
         }
 }
 
+static void
+print_bad_arguments_message (const char *program_name)
+{
+	fprintf (stderr, "%s: invalid arguments.\nTry `%s' --help for more information.\n",
+		program_name, program_name);
+	exit (-1);
+}
+
+static void
+parse_arguments (int argc,
+		 char **argv,
+		 const struct poptOption *popt_options,
+		 poptContext *popt_context)
+{
+	int rc;
+
+ 	*popt_context = poptGetContext (argv[0], argc, argv, popt_options, 0);
+
+	if (-1 != (rc = poptGetNextOpt (*popt_context))) {
+		print_bad_arguments_message (argv[0]);
+		exit (-1);
+	}
+	
+	if (opt_help) {
+		poptPrintUsage (*popt_context, stdout, 0);
+		exit (0);
+	}
+}
